@@ -567,6 +567,12 @@ if hasattr(c, "LabApp"):
 import notebook.dataiku.kernelmanager
 c.MultiKernelManager.kernel_manager_class = "notebook.dataiku.kernelmanager.DataikuIOLoopKernelManager"
 
+from notebook.dataiku.dipproperties import DipProperties, SHARED_CONTEXT
+if DipProperties().get(SHARED_CONTEXT) == 'true':
+	c.MultiKernelManager.shared_context = True
+else:
+	c.MultiKernelManager.shared_context = False
+
 #------------------------------------------------------------------------------
 # MappingKernelManager(MultiKernelManager) configuration
 #------------------------------------------------------------------------------
