@@ -1203,6 +1203,7 @@ app.controller('BuildMultipleComputablesController', function ($scope, JobDefini
     };
 
     const WT1BuildMultipleComputablesEvents = {
+        BuildFlowZoneStart : 'flow-zone-build-computable-start',
         BuildComputableFindOutputsStart : 'flow-build-computable-find-outputs-start',
         RunRecipeFindOutputsStart : 'flow-run-recipe-find-outputs-start',
         FindOuputsPreview:'flow-build-computable-find-outputs-preview',
@@ -1244,7 +1245,7 @@ app.controller('BuildMultipleComputablesController', function ($scope, JobDefini
 
         if (originalBuildContext.origin === "COMPUTABLE") {
             ret.computableType = originalBuildContext.computable.type;
-        } else {
+        } else if (originalBuildContext.origin === "RECIPE") {
             ret.recipeType = originalBuildContext.recipe.type;
         }
         return ret
@@ -1254,9 +1255,16 @@ app.controller('BuildMultipleComputablesController', function ($scope, JobDefini
     $scope.startJob = function() {
         $scope.isBuildingDataset = true;
 
-        let wt1Label;
-        if ($scope.originalBuildContext) {
-            wt1Label = $scope.originalBuildContext.origin === "COMPUTABLE" ? WT1BuildMultipleComputablesEvents.BuildComputableFindOutputsStart : WT1BuildMultipleComputablesEvents.RunRecipeFindOutputsStart;
+        let wt1Label = undefined;
+        if ($scope.originalBuildContext?.origin === "COMPUTABLE") {
+            wt1Label = WT1BuildMultipleComputablesEvents.BuildComputableFindOutputsStart;
+        } else if ($scope.originalBuildContext?.origin === "FLOW_ZONE") {
+            wt1Label = WT1BuildMultipleComputablesEvents.BuildFlowZoneStart;
+        } else if ($scope.originalBuildContext?.origin === "RECIPE") {
+            wt1Label = WT1BuildMultipleComputablesEvents.RunRecipeFindOutputsStart;
+        }
+
+        if (wt1Label) {
             WT1.tryEvent(wt1Label, () => createBuildWT1Event($scope.originalBuildContext));
         }
 
@@ -1268,7 +1276,7 @@ app.controller('BuildMultipleComputablesController', function ($scope, JobDefini
             setErrorInScope.bind($scope)(data, status, headers)
             $scope.isBuildingDataset = false;
 
-            if ($scope.originalBuildContext) {
+            if (wt1Label) {
                 wt1Label = `${wt1Label}-failed`;
                 WT1.tryEvent(wt1Label, () => createBuildWT1Event($scope.originalBuildContext));
             }
