@@ -38,7 +38,7 @@
                 newPage.displayedTitle = $scope.interpolatedTitle;
                 newPage.id = null;
                 if (insights) {
-                    TileUtils.getFlattenTileList(newPage.grid.tiles).forEach(function(tile) {
+                    TileUtils.getFlattenedTileList(newPage.grid.tiles).forEach(function(tile) {
                         if (tile.tileType == 'INSIGHT') {
                             const insight = insights.shift();
                             tile.insightId = insight.id;
@@ -56,7 +56,7 @@
                     const insightIds = [];
                     const insightNames = [];
 
-                    TileUtils.getFlattenTileList($scope.page.grid.tiles).forEach(function(tile) {
+                    TileUtils.getFlattenedTileList($scope.page.grid.tiles).forEach(function(tile) {
                         if (tile.tileType == 'INSIGHT') {
                             insightIds.push(tile.insightId);
                             insightNames.push($scope.insightsMap[tile.insightId].name);
