@@ -3,13 +3,15 @@
 
 var app = angular.module('dataiku.datasets');
 
-app.controller("BaseSQLDatasetController", function($scope, $stateParams, $controller, DataikuAPI, withConsistency, SqlConnectionNamespaceService) {
+app.controller("BaseSQLDatasetController", function($scope, $stateParams, $controller, DataikuAPI, withConsistency, SqlConnectionNamespaceService,
+        DatasetTypesService) {
+
     $controller("BaseRowDatasetController", {$scope: $scope, withConsistency: withConsistency});
 
     $scope.partitionsList = {};
 
     $scope.isCatalogAware = function() {
-        return $scope.dataset.type && ['Databricks', 'BigQuery', 'Snowflake', 'SQLServer', 'JDBC', 'Trino'].includes($scope.dataset.type);
+        return $scope.dataset.type && ['Databricks', 'BigQuery', 'Snowflake', 'SQLServer', 'JDBC', 'Trino', 'FabricWarehouse'].includes($scope.dataset.type);
     };
 
     $scope.isSchemaAware = function() {
@@ -19,15 +21,16 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
     $scope.hooks = {};
 
     $scope.setConnections = function(data) {
-        $scope.connections = data;
+        $scope.connections = data.map(connection => connection['name']);
+        $scope.connectionsWithDescriptions = data;
         if (!$scope.dataset.params.connection && data.length) {
-            $scope.dataset.params.connection = data[0];
+            $scope.dataset.params.connection = data[0]['name'];
             $scope.test(false, false);
         }
     };
 
     $scope.hooks.getConnectionNames = function() {
-        return DataikuAPI.connections.getNames($scope.dataset.type).success(function (data) {
+        return DataikuAPI.connections.getTypeAndNames($scope.dataset.type).success(function (data) {
             $scope.setConnections(data);
         }).error(setErrorInScope.bind($scope));
     };
@@ -78,7 +81,7 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
         });
     };
 
-    $scope.uiState = { codeSamplesSelectorVisible: false };
+    $scope.uiState = { codeSamplesSelectorVisible: false, useExplicitPartitionsList: false };
 
     /* Need to manually refresh code mirror on show.
      * as it will compute various dimensions and set them.
@@ -97,10 +100,14 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
     $scope.isPartitioned = function() {
         return $scope.dataset && $scope.dataset.partitioning && $scope.dataset.partitioning.dimensions && $scope.dataset.partitioning.dimensions.length > 0;
     };
-    // we resolved the "forbidPartitionsWriteToNonPartitionedTable" setting, if it is set to "INHERIT" then the setting at the connection level need to be checked
-    $scope.isPartitionsWriteToNonPartitionedTableForbidden = function() {
-        return $scope.dataset.params.forbidPartitionsWriteToNonPartitionedTable === "ENABLED" ||
-                    $scope.dataset.params.forbidPartitionsWriteToNonPartitionedTable === "INHERIT" && $scope.datasetFullInfo && $scope.datasetFullInfo.connectionForbidPartitionsWriteToNonPartitionedTable;
+    // we resolved the "forbidPartitionsWriteToNonPartitionedTable" setting, if it is set to "INHERIT" then the setting at the connection level needs to be checked
+    $scope.isPartitioningConsistencyEnabled = function() {
+        return (
+            $scope.dataset.params.forbidPartitionsWriteToNonPartitionedTable === "ENABLED" ||
+            ($scope.dataset.params.forbidPartitionsWriteToNonPartitionedTable === "INHERIT" &&
+                $scope.datasetFullInfo &&
+                $scope.datasetFullInfo.connectionForbidPartitionsWriteToNonPartitionedTable)
+        );
     };
     $scope.$watch("dataset.params.partitioned + dataset.params.mode", function() {
         if ($scope.isPartitioned()) {
@@ -118,43 +125,100 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
         }
     };
 
-    $scope.shouldDisplayBigQueryPartitionMismatchWarning = function() {
-        if (!$scope.dataset.managed) {
+    $scope.createBigQueryDatePartitioningPeriods = function(partitioningColumn) {
+        if (partitioningColumn && DatasetTypesService.isTemporalType(partitioningColumn.type)) {
+            const result = [ [ "YEAR", "By year" ], [ "MONTH", "By month" ], [ "DAY", "By day" ] ];
+            if (!DatasetTypesService.isDateOnly(partitioningColumn.type)) {
+                result.push([ "HOUR", "By hour" ]);
+            }
+            return result;
+        } else {
+            return [];
+        }
+    };
+
+    $scope.$watch("dataset.params.bigQueryPartitioningField", function(newValue) {
+        const partitioningColumn = $scope.dataset.schema?.columns?.find(c => c.name === newValue);
+        $scope.bigQueryDatePartitioningPeriods = $scope.createBigQueryDatePartitioningPeriods(partitioningColumn);
+    });
+
+    /* For autocompletion in primary / distribute / sort / ... keys
+       Add for checking if partition columns exist in the dataset
+    */
+    $scope.$watch("dataset.schema", function(nv, ov){
+        if (!nv) return;
+        $scope.schemaColumnNames = nv.columns ? nv.columns.map(x => x.name) : [];
+    }, true);
+
+    $scope.shouldWarnPartitionColumnMissing = function(columnName) {
+        return $scope.schemaColumnNames && $scope.schemaColumnNames.length > 0 && !$scope.schemaColumnNames.includes(columnName);
+    }
+
+    $scope.shouldDisplayBigQueryPartitionMismatchMessage = function() {
+        if (!$scope.dataset.managed || $scope.dataset.type !== 'BigQuery' || !$scope.isPartitioned()) {
             return false;
         }
-        if ($scope.dataset.type === 'BigQuery') {
-            if ($scope.isPartitioned()) {
-                if (!$scope.dataset.params.useBigQueryPartitioning) {
-                    // if the native partitioning is not set but 'Partition Consistency' is enabled, we will display an error instead as recipes will fail
-                    return !$scope.isPartitionsWriteToNonPartitionedTableForbidden();
-                }
-                // dataset partitioning column and table partitioning column do not match
-                if ($scope.dataset.params.bigQueryPartitioningField !== $scope.dataset.partitioning.dimensions[0].name) {
-                    return true;
-                }
-                // dataset is partitioned by discrete dimension but BigQuery table partitioning is not configured by discrete
-                if ($scope.dataset.params.bigQueryPartitioningType === 'RANGE' && $scope.dataset.partitioning.dimensions[0].type !== 'value') {
-                    return true;
-                }
-                // dataset is partitioned by day but BigQuery table partitioning is not configured by day
-                if ($scope.dataset.params.bigQueryPartitioningType === 'DATE' && $scope.dataset.partitioning.dimensions[0].type !== 'time') {
-                    return true;
-                }
-            }
+        // BigQuery table partitioning is not set
+        if (!$scope.dataset.params.useBigQueryPartitioning) {
+            return true;
+        }
+        // dataset partitioning column and BigQuery table partitioning column do not match
+        if ($scope.dataset.params.bigQueryPartitioningField !== $scope.dataset.partitioning.dimensions[0].name) {
+            return true;
+        }
+        // dataset is partitioned by discrete dimension but BigQuery table partitioning is not configured by discrete
+        if ($scope.dataset.params.bigQueryPartitioningType === 'RANGE' && $scope.dataset.partitioning.dimensions[0].type !== 'value') {
+            return true;
+        }
+        // dataset is partitioned by date but BigQuery table partitioning is not configured by date
+        if ($scope.dataset.params.bigQueryPartitioningType === 'DATE' && $scope.dataset.partitioning.dimensions[0].type !== 'time') {
+            return true;
+        }
+        // both dataset and BigQuery table are partitioned by date but the dataset dimension period and the BigQuery table partitioning period do not match
+        if (
+            $scope.dataset.params.bigQueryPartitioningType === 'DATE' &&
+            $scope.dataset.partitioning.dimensions[0].params?.period !== $scope.dataset.params.bigQueryPartitioningPeriod
+        ) {
+            return true;
         }
         return false;
     }
 
+    $scope.$watch("dataset.params.explicitPartitionsList", function(newValue) {
+        if (newValue !== undefined && !$scope.uiState.useExplicitPartitionsList) {
+            $scope.uiState.useExplicitPartitionsList = true;
+        }
+        if ($scope.uiState.explicitPartitionsList === undefined) {
+            $scope.uiState.explicitPartitionsList = newValue;
+        }
+    });
+
+    $scope.$watch("uiState.explicitPartitionsList", function(newValue) {
+        $scope.dataset.params.explicitPartitionsList = $scope.uiState.explicitPartitionsList;
+    });
+
+    $scope.useExplicitPartitionsList = function(use) {
+        if (use) {
+            if ($scope.dataset.params.explicitPartitionsList === undefined) {
+                $scope.dataset.params.explicitPartitionsList = "";
+            }
+            if ($scope.uiState.explicitPartitionsList && !$scope.dataset.params.explicitPartitionsList) {
+                $scope.dataset.params.explicitPartitionsList = $scope.uiState.explicitPartitionsList;
+            }
+        } else {
+            delete $scope.dataset.params.explicitPartitionsList;
+        }
+    }
+
     $scope.removeDimension = function(index) {
         $scope.dataset.partitioning.dimensions.splice(index, 1);
     };
 
     $scope.addDimension = function() {
-        const defaultTimeDimensionPeriod = $scope.dataset.type === 'BigQuery' ? "DAY": $scope.timeDimensionPeriods[0];
         $scope.dataset.partitioning.dimensions.push({
             name : 'dimension_' + $scope.dataset.partitioning.dimensions.length,
             type : 'value',
-            params : {'period': defaultTimeDimensionPeriod }
+            params : {'period': $scope.timeDimensionPeriods[0] }
         });
     };
 
@@ -175,7 +239,8 @@ app.controller("BaseSQLDatasetController", function($scope, $stateParams, $contr
 });
 
 
-app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $controller, LoggerProvider, DataikuAPI, DatasetUtils, Dialogs, $state) {
+app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $controller, LoggerProvider, 
+    DataikuAPI, DatasetUtils, Dialogs, ActivityIndicator, $state) {
     $controller("BaseSQLDatasetController", {$scope: $scope, withConsistency: true});
 
     var Logger = LoggerProvider.getLogger('datasets.sql');
@@ -224,13 +289,14 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
         $scope.test(false, false);
     };
 
+    // TODO : check if this function is still in use
     $scope.gotoSearchAndImport = function () {
         $scope.uiState.bypassDirtinessCheck = true;
         DataikuAPI.connections.countIndexedAndUnindexed().success(function (data) {
             if (data.indexedConnections > 0) {
                 $state.go('projects.project.datacatalog.datasources', {selectedTab: 'external-tables'});
             } else {
-                $state.go("projects.project.datacatalog.connectionexplorer");
+                $state.go("projects.project.datacatalog.database-explorer");
             }
 
         }).error(setErrorInScope.bind($scope));
@@ -252,12 +318,99 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
             "which offers ability to filter, search and mass import</p>").then($scope.test.bind(this, true, false))
     }
 
-    function addDimensionIfNecessary(dimensions, dimensionToAdd) {
-        if (!dimensions.some(d => angular.equals(d, dimensionToAdd))) {
-            dimensions.push(dimensionToAdd);
+
+    $scope.shouldWarnAboutTimezoneMismatch = function(dimension) {
+        // Only called for BigQuery external datasets
+        if (dimension.type === "time") {
+            // Simple check of whether assumed timezone is UTC (may miss some cases - then we will still warn, but not a big problem)
+            const utcEquivalents = ["UTC", "Etc/UTC", "GMT", "Etc/GMT"];
+            const assumedTzIsUtc = utcEquivalents.includes($scope.dataset.params.assumedTzForUnknownTz);
+            if (assumedTzIsUtc) {
+                return false;
+            }
+            const columns = $scope.dataset.schema?.columns;
+            if (columns) {
+                const column = columns.find((column) => column.name === dimension.name);
+                if (column) {
+                    // Check if the date time read mode could shift the partition boundaries for this column
+                    // See BigQueryUtils.partitionSchemeMatches() in Java for rough equivalent
+                    return (column.originalSQLType === "DATETIME" && $scope.dataset.params.datetimenotzReadMode === "AS_DATE")
+                            || (column.originalSQLType === "DATE" && $scope.dataset.params.dateonlyReadMode === "AS_DATE");
+                }
+            } else {
+                    // The schema hasn't loaded yet (or there really are no columns)
+                    // Probably means they are setting up partition dimensions before clicking Test Connection
+                    // Not sure how often this happens - still  worth warning, but this check is not as precise as when we have a schema
+                    return $scope.dataset.params.datetimenotzReadMode === "AS_DATE" || $scope.dataset.params.dateonlyReadMode === "AS_DATE";
+            }
+        }
+        return false;
+    }
+
+    $scope.partitionsValidForBigQuery = function() {
+        if ($scope.dataset.params.externalPartitionField) {
+            const dimensions = $scope.dataset.partitioning?.dimensions || [];
+            if (dimensions.length === 0) {
+                // Only true when partitioning is deactivated 
+                // we return to stop console errors but the value does not effect the UI (and this is considered a valid config)
+                return true;
+            }
+
+            const bqDimInFirstPosition = $scope.dataset.partitioning.dimensions[0].name === $scope.dataset.params.externalPartitionField;
+            const inAnyOtherPosition = $scope.dataset.partitioning.dimensions.slice(1).some((dimension) => dimension.name === $scope.dataset.params.externalPartitionField);
+            $scope.bqPartitionMissing = !bqDimInFirstPosition && !inAnyOtherPosition;
+            return bqDimInFirstPosition && !inAnyOtherPosition;
+        } else {
+            return true;
         }
     }
 
+    // When doing Test Connection on BigQuery, ensure the dimensions are compatible with the BQ partition dimension
+    // Add the bqDimension if necessary, delete any others relating to the same column
+    function fixDimensionsForBQIfNecessary(dssDimensions, bqDimension) {
+        function dimensionsEqual(dim1, dim2) {
+            // Check equality ignoring integerRangeInfo
+            if (dim1.name !== dim2.name || dim1.type !== dim2.type) {
+                return false;
+            }
+            if (dim1.type === "time") {
+                return dim1.params?.period === dim2.params?.period;
+            }
+            return true;
+        }
+
+        const dimensionsOfSameName = [];
+        const dimensionsOfSameNameIndexes = [];
+        for (let i = 0; i < dssDimensions.length; ++i) {
+            if (dssDimensions[i].name === bqDimension.name) {
+                dimensionsOfSameName.push(dssDimensions[i]);
+                dimensionsOfSameNameIndexes.push(i);
+            }
+        }
+
+        // If there is one matching dimension, it is the first and is equal to the BQ one, all good, no action needed
+        // Otherwise we have to fix things
+        const hasMatchingFirstCorrectDimension = dimensionsOfSameName.length === 1
+                && dimensionsOfSameNameIndexes[0] === 0
+                && dimensionsEqual(dimensionsOfSameName[0], bqDimension);
+
+        if (!hasMatchingFirstCorrectDimension) {
+            if (dimensionsOfSameName.length === 0) {
+                dssDimensions.unshift(bqDimension);
+                ActivityIndicator.info("A partition dimension has been added to match that of the BigQuery table", 5000);
+            } else {
+                // Delete all dimensions of that name
+                for (let i = dimensionsOfSameNameIndexes.length - 1; i >= 0; --i) {
+                    dssDimensions.splice(dimensionsOfSameNameIndexes[i], 1);
+                }
+                // Add correct BQ dimension at the start
+                dssDimensions.unshift(bqDimension);
+                ActivityIndicator.info("Some partition dimensions have been adjusted or removed to match the BigQuery table", 5000);
+            }
+        }
+    }
+
+
     $scope.test = function (listTables, testTableOrQuery) {
         var previousTableList = $scope.testResult == null ? null : $scope.testResult.tablesList;
         const getTableMetadata = !listTables && !previousTableList && $scope.isNewDataset &&
@@ -283,19 +436,19 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
                 $scope.consistency = { empty : false, result : $scope.testResult.schemaDetection };
                 $scope.consistency.kind = DatasetUtils.getKindForConsistency($scope.dataset);
                 $scope.dataset.schema = $scope.testResult.schemaDetection.newSchema;
-                let addPartition = false;
+                // For BigQuery we ensure the partition dimension in the external table is present as the first DSS partition dimension
+                let ensureExternalPartitionDimension = false;
                 if ($scope.dataset.type === 'BigQuery') {
                     $scope.dataset.params.externalPartitionField =
                         $scope.testResult.tablePartitionInfo &&
                         $scope.testResult.tablePartitionInfo.dimension &&
                         $scope.testResult.tablePartitionInfo.dimension.name;
-                    addPartition = ($scope.dataset.params.externalPartitionField != null);
+                    ensureExternalPartitionDimension = ($scope.dataset.params.externalPartitionField != null);
                 }
-                if (addPartition) {
+                if (ensureExternalPartitionDimension) {
                     $scope.dataset.partitioning = $scope.dataset.partitioning || {};
                     $scope.dataset.partitioning.dimensions = $scope.dataset.partitioning.dimensions || [];
-                    // If the dimension already exists, don't add it again. Only check for exact duplicates (like from clicking "Test" multiple times).
-                    addDimensionIfNecessary($scope.dataset.partitioning.dimensions, $scope.testResult.tablePartitionInfo.dimension);
+                    fixDimensionsForBQIfNecessary($scope.dataset.partitioning.dimensions, $scope.testResult.tablePartitionInfo.dimension);
                 }
             }
             if ($scope.testResult.preview) {
@@ -375,7 +528,6 @@ app.controller("ExternalSQLDatasetController", function($scope, $stateParams, $c
         // thing. We NEVER run it on simple model change, and in the specific case
         // of SQL, we even NEVER run it without explicit user action.
     }, true);
-
 });
 
 app.controller("ExternalHiveDatasetController", function($scope, $stateParams, $controller, LoggerProvider, DataikuAPI, DatasetUtils) {
@@ -463,6 +615,12 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
         // On managed, we more or less guarantee that we only target a table, so it can't be too heavy
         // EXCEPT IN CASE OF PARTITIONING
         $scope.test(false, !$scope.dataset.params.partitioned);
+
+        // init warning on name duplicate
+        return DataikuAPI.datasets.managedSQL.testDatasetStorageConflicts($scope.dataset.projectKey, $scope.dataset).then(({data}) => {
+            $scope.testDatasetStorageConflicts = data;
+            $scope.origTestDatasetStorageConflicts = data;
+        }).catch(setErrorInScope.bind($scope));
     };
 
     $scope.dropTable = function () {
@@ -507,7 +665,9 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
             || c.type === 'bigint' || c.type === 'int' || c.type === 'smallint' || c.type === 'tinyint'));
 
         // Remove the column used to partition the table
-        result = result.filter(c => (c.name !== $scope.dataset.params.bigQueryPartitioningField));
+        if ($scope.dataset.params.useBigQueryPartitioning && $scope.dataset.params.bigQueryPartitioningField) {
+            result = result.filter(c => (c.name !== $scope.dataset.params.bigQueryPartitioningField));
+        }
 
         // Remove the columns used to cluster the table above.
         for (let i = 0; i < clusteringIndex; i++) {
@@ -529,7 +689,7 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
         $scope.dataset.params.bigQueryClusteringColumns.splice(index, 1);
     };
 
-    $scope.forbidBigQueryWritePartitionsOptions = [
+    $scope.inheritableEnabledSettingOptions = [
         { n: 'Enabled', v: 'ENABLED' },
         { n: 'Disabled', v: 'DISABLED' },
         { n: 'Inherit connection settings', v: 'INHERIT' }
@@ -540,53 +700,67 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
     }
 
     /**
-    *    Automatically adjust a BigQuery dataset partitioning if it's modified and partitioned with a single partitioning column of type time-based with granularity DAY
-    **/
+     * Automatically adjust a BigQuery dataset time-based partitioning when the DSS partitioning is modified, to match the first DSS dimension
+     * Discrete partitioning is ignored in this, and only the first dimension on the DSS side is considered
+     */
     $scope.$watch("dataset.partitioning", function(newPart, oldPart) {
         const isModified = newPart !== oldPart;
-        if ($scope.dataset.type === 'BigQuery' && isModified) {
-            const bigQueryPartitioningEnabled = $scope.dataset?.params?.useBigQueryPartitioning && $scope.dataset?.params?.bigQueryPartitioningField;
-            const isSingleDimension = $scope.dataset.partitioning.dimensions.length === 1;
-            if ($scope.isPartitioned() && !bigQueryPartitioningEnabled && isSingleDimension) {
-                const dimension = $scope.dataset.partitioning.dimensions[0];
-                const isSingleDayTimeDimension = dimension.type === 'time' && dimension.params?.period === 'DAY';
-                const schemaColumn = $scope.dataset.schema.columns.find(col => col.name === dimension.name);
-                if (isSingleDayTimeDimension && schemaColumn && isColumnTemporal(schemaColumn)) {
-                    Object.assign($scope.dataset.params, {
-                        useBigQueryPartitioning: true,
-                        bigQueryPartitioningField: dimension.name,
-                        bigQueryPartitioningType: "DATE"
-                    });
-                    ActivityIndicator.info("Native BigQuery partitioning has been enabled (see the Advanced tab)", 5000);
+        const dssHasDimensions = ($scope.dataset.partitioning?.dimensions?.length || 0) >= 1;
+        if ($scope.dataset.type !== 'BigQuery' || !isModified || !dssHasDimensions) {
+            return;
+        }
+        const dimension = $scope.dataset.partitioning.dimensions[0];
+        const schemaColumn = $scope.dataset.schema.columns.find(col => col.name === dimension.name);
+        const validPartitioningPeriods = $scope.createBigQueryDatePartitioningPeriods(schemaColumn).map((x) => x[0]);
+        if (dimension.type === "time" && validPartitioningPeriods.includes(dimension.params?.period) && isColumnTemporal(schemaColumn)) {
+            const bigQueryPartitioningEnabled = $scope.dataset.params?.useBigQueryPartitioning && $scope.dataset.params?.bigQueryPartitioningField;
+            if (bigQueryPartitioningEnabled) {
+                if (
+                    $scope.dataset.params.bigQueryPartitioningField === dimension.name &&
+                    $scope.dataset.params.bigQueryPartitioningType === "DATE" &&
+                    $scope.dataset.params.bigQueryPartitioningPeriod !== dimension.params.period
+                ) {
+                    $scope.dataset.params.bigQueryPartitioningPeriod = dimension.params.period;
+                    ActivityIndicator.info("Native BigQuery partitioning period has been updated (see the Advanced tab)", 5000);
                 }
+            } else {
+                Object.assign($scope.dataset.params, {
+                    useBigQueryPartitioning: true,
+                    bigQueryPartitioningField: dimension.name,
+                    bigQueryPartitioningType: "DATE",
+                    bigQueryPartitioningPeriod: dimension.params.period
+                });
+                ActivityIndicator.info("Native BigQuery partitioning has been enabled (see the Advanced tab)", 5000);
             }
         }
     }, true);
 
-    $scope.$watchGroup(["dataset.partitioning.dimensions", "dataset.params.useBigQueryPartitioning"], function([newDim, newPart], [oldDim, oldPart]) {
-        const anyChange = newDim !== oldDim || newPart !== oldPart;
-        if (anyChange && $scope.dataset?.type === 'BigQuery') {
-            // Test again for BigQuery whenever we enable/disable partitioning to surface any remote table partitioning mismatch
-            $scope.test(false);
-        }
-    });
+    $scope.$watchGroup(
+        ["dataset.params.useBigQueryPartitioning", "dataset.params.bigQueryPartitioningField", "dataset.params.bigQueryPartitioningPeriod"],
+        function([newPart, newField, newPeriod], [oldPart, oldField, oldPeriod]) {
+            if (newField) {
+                const partitioningColumn = $scope.dataset.schema.columns.find(c => c.name === newField);
+                $scope.dataset.params.bigQueryPartitioningType = (!partitioningColumn || DatasetTypesService.isTemporalType(partitioningColumn.type))
+                    ? "DATE"
+                    : "RANGE";
 
-
-    $scope.$watch("dataset.params.bigQueryPartitioningField", function(nv, ov) {
-        if (nv) {
-            const partitioningColumn = $scope.dataset.schema.columns.find(c => c.name === nv);
-            $scope.dataset.params.bigQueryPartitioningType = (!partitioningColumn || DatasetTypesService.isTemporalType(partitioningColumn.type)) ? "DATE" : "RANGE";
-
-            // If the new value was selected as a partitioning column, remove it from clustering columns if it was used as a clustering column
-            const clusteringColumn = $scope.dataset.params.bigQueryClusteringColumns && $scope.dataset.params.bigQueryClusteringColumns.find(c => c.name === nv);
-            if (clusteringColumn) {
-                const index = $scope.dataset.params.bigQueryClusteringColumns.indexOf(clusteringColumn);
-                $scope.dataset.params.bigQueryClusteringColumns.splice(index, 1);
+                // If the new value was selected as a partitioning column, remove it from clustering columns if it was used as a clustering column
+                const clusteringColumn = (
+                    $scope.dataset.params.bigQueryClusteringColumns &&
+                    $scope.dataset.params.bigQueryClusteringColumns.find(c => c.name === newField)
+                );
+                if (clusteringColumn) {
+                    const index = $scope.dataset.params.bigQueryClusteringColumns.indexOf(clusteringColumn);
+                    $scope.dataset.params.bigQueryClusteringColumns.splice(index, 1);
+                }
+            }
+            const anyChanged = newPart !== oldPart || newField !== oldField || newPeriod !== oldPeriod;
+            if (anyChanged) {
+                // Test again when BigQuery native partitioning is changed so warnings are correctly raised
+                $scope.test(false);
             }
-            // Test again when BigQuery native partitioning is changed so warning are correctly raised
-            $scope.test(false);
         }
-    });
+    );
 
     $scope.$watch("dataset.params.bigQueryClusteringColumns", function(nv, ov) {
         if (nv) {
@@ -622,9 +796,9 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
                 $scope.testResultCase = 'CONNECTION_TABLE_DOES_NOT_EXIST';
             } else if (!$scope.testResult.preview) {
                 $scope.testResultCase = 'CONNECTION_TABLE_PREVIEW_FAILED';
-            } else if ($scope.isPartitioned() && !$scope.testResult.tableIsPartitioned && $scope.isPartitionsWriteToNonPartitionedTableForbidden()) {
+            } else if (!$scope.testResult.datasetPartitioningMatchesTable && $scope.isPartitioningConsistencyEnabled()) {
                 $scope.testResultCase = 'CONNECTION_PARTITIONING_MISMATCH_FORBIDDEN';
-            } else if ($scope.testResult.partitioningMismatch) {
+            } else if (!$scope.testResult.nativePartitioningMatchesTable) {
                 $scope.testResultCase = 'CONNECTION_PARTITIONING_MISMATCH_WARNING';
             } else if (!$scope.testResult.schemaMatchesTable) {
                 $scope.testResultCase = 'CONNECTION_SCHEMA_MISMATCH';
@@ -647,18 +821,41 @@ app.controller("ManagedSQLDatasetController", function($scope, $stateParams, $co
         return deferred.promise;
     };
 
+    const superSaveDataset = $scope.saveDataset;
+    $scope.saveDataset = () => {
+        if($scope.origDataset?.params && (
+            $scope.origDataset.params.connection !== $scope.dataset.params.connection
+            || $scope.origDataset.params.catalog !== $scope.dataset.params.catalog
+            || $scope.origDataset.params.schema !== $scope.dataset.params.schema
+            || $scope.origDataset.params.table !== $scope.dataset.params.table
+        )) {
+            let checkResult;
+            return DataikuAPI.datasets.managedSQL.testDatasetStorageConflicts($scope.dataset.projectKey, $scope.dataset)
+                .then(({data}) => {
+                    checkResult = data;
+                    if (checkResult.hasDuplicate) {
+                        return CreateModalFromTemplate("/templates/datasets/modals/duplicate-table-name-warning-modal.html", $scope, null, function(modalScope) {
+                            modalScope.otherDatasetName = checkResult.duplicateName;
+                            modalScope.conflictingLabel = checkResult.conflictingLabel;
+                        });
+                    }
+                }).then(
+                    superSaveDataset
+                ).finally(() => {
+                    $scope.testDatasetStorageConflicts = checkResult;
+                });
+        } else {
+            $scope.testDatasetStorageConflicts = $scope.origTestDatasetStorageConflicts; // restore the original state in case it was changed / tried to save / cancel on warning modal / restored
+            return superSaveDataset();
+        }
+    }
+
     $scope.$watch("dataset.params.tableCreationMode", function(nv, ov) {
         if (nv == "custom" && ov == "auto" && !$scope.dataset.params.customCreateStatement) {
              $scope.dataset.params.customCreateStatement = $scope.testResult.autogeneratedCreateStatement;
         }
     });
 
-    /* For autocompletion in primary / distribute / sort / ... keys */
-    $scope.$watch("dataset.schema", function(nv, ov){
-        if (!nv) return;
-        $scope.schemaColumnNames = nv.columns.map(x => x.name);
-    }, true);
-
     /* Fixup everything we can ... */
     $scope.$watch('dataset.params', function (nv, ov) {
     }, true);
