@@ -13,7 +13,7 @@
     /**
      * A set of helpers to create and enhance d3 svg axes
      */
-    function D3ChartAxes(d3Utils, CHART_AXIS_TYPES, CHART_TYPES, CHART_MODES, ChartDimension, ChartUADimension, ChartAxesUtils, AxisTicksConfigMode, ChartFeatures, ChartFormatting, ChartsStaticData, AxisTicksConfiguration, ChartYAxisPosition, ChartFormattingPaneSections, GridlinesAxisType, ChartDataUtils, ChartLabels) {
+    function D3ChartAxes(d3Utils, CHART_AXIS_TYPES, CHART_TYPES, CHART_MODES, ChartDimension, ChartUADimension, ChartAxesUtils, AxisTicksConfigMode, ChartFeatures, ChartFormatting, ChartsStaticData, AxisTicksConfiguration, ChartYAxisPosition, ChartFormattingPaneSections, GridlinesAxisType, ChartDataUtils, ChartLabels, ChartMeasure, CHART_VARIANTS, ValuesInChartPlacementMode) {
 
         const addFormatterToBinnedAxis = (axis, tickExtents, formattingOptions) => {
             axis.tickFormat(ChartFormatting.getForBinnedAxis(tickExtents, formattingOptions));
@@ -23,10 +23,10 @@
             axis.tickFormat(ChartFormatting.getForCustomBinnedAxis(tickExtents, formattingOptions));
         };
 
-        const addFormatterToOrdinalAxis = (axis, tickExtents) => {
+        const addFormatterToOrdinalAxis = (axis, tickExtents, formattingOptions) => {
             axis.tickFormat((_d, i) => {
                 const value = tickExtents[i];
-                return ChartFormatting.getForOrdinalAxis(value);
+                return ChartFormatting.getForOrdinalAxis(value, formattingOptions);
             });
         };
 
@@ -81,10 +81,10 @@
                 .attr('opacity', 0)
                 .attr('font-size', `${fontSize}px`)
                 .text('0');
-            if (textElement){
+            if (textElement) {
                 const textHeight = textElement.node()?.getBBox().height;
                 textElement.remove();
-                if (textHeight){
+                if (textHeight) {
                     result = textHeight;
                 }
             }
@@ -106,7 +106,7 @@
              * @param {ChartDef.java}              chartDef
              * @returns {d3 axis}
              */
-            createAxis: function(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions, chartDef, ignoreLabels) {
+            createAxis: function(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions, chartDef, ignoreLabels, otherAxesIndexes = []) {
                 if (!axisSpec || !formattingOptions) {
                     return null;
                 }
@@ -118,7 +118,7 @@
                         axis = svc.createDimensionAxis(chartData, axisSpec, isLogScale, includeZero, formattingOptions, chartDef, ignoreLabels);
                         break;
                     case CHART_AXIS_TYPES.MEASURE:
-                        axis = svc.createMeasureAxis(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions);
+                        axis = svc.createMeasureAxis(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions, otherAxesIndexes);
                         break;
                     default:
                         throw new Error('Unknown axis type: ' + axisSpec.type);
@@ -145,6 +145,7 @@
              * @returns {d3 axis}
              */
             createDimensionAxis: function(chartData, axisSpec, isLogScale, includeZero, formattingOptions, chartDef, ignoreLabels) {
+                const finalFormattingOptions = { ...formattingOptions, ...chartData?.getSpecialValuesLabelsOverrides?.() };
                 const isNumerical = ChartAxesUtils.isNumerical(axisSpec);
                 let extent = ChartAxesUtils.getDimensionExtent(chartData, axisSpec, ignoreLabels);
 
@@ -179,25 +180,25 @@
                         axis = d3.svg.axis().scale(linearScale);
                         axis.scaleType = SCALE_TYPES.LINEAR;
                     }
-                    svc.addNumberFormatterToAxis(axis, formattingOptions);
+                    svc.addNumberFormatterToAxis(axis, finalFormattingOptions);
                     return axis;
                 };
 
                 const createOrdinalAxis = (isGroupedNumerical = false, isCustomBinning = false) => {
                     const axis = d3.svg.axis().scale(ordinalScale);
                     if (isCustomBinning) {
-                        addFormatterToCustomBinnedAxis(axis, extent.values, formattingOptions);
+                        addFormatterToCustomBinnedAxis(axis, extent.values, finalFormattingOptions);
                     } else if (isGroupedNumerical) {
-                        addFormatterToBinnedAxis(axis, extent.values, formattingOptions);
+                        addFormatterToBinnedAxis(axis, extent.values, finalFormattingOptions);
                     } else {
-                        addFormatterToOrdinalAxis(axis, extent.values);
+                        addFormatterToOrdinalAxis(axis, extent.values, finalFormattingOptions);
                     }
                     axis.scaleType = SCALE_TYPES.ORDINAL;
                     return axis;
                 };
 
                 const createTimeAxis = () => {
-                    timeScale = d3.time.scale().domain([extent.min, extent.max]);
+                    timeScale = d3.time.scale.utc().domain([extent.min, extent.max]);
                     const timeAxis = d3.svg.axis().scale(timeScale);
                     timeAxis.scaleType = SCALE_TYPES.TIME;
                     return timeAxis;
@@ -290,8 +291,8 @@
              * @param {{multiplier: keyof ChartsStaticData.availableMultipliers, decimalPlaces: number, prefix: string, suffix: string }}    formattingOptions
              * @returns {d3 axis}
              */
-            createMeasureAxis: function(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions) {
-                axisSpec.extent = ChartAxesUtils.getMeasureExtent(chartData, axisSpec, isLogScale, includeZero, true);
+            createMeasureAxis: function(chartData, axisSpec, isPercentScale, isLogScale, includeZero, formattingOptions, otherAxesIndexes = []) {
+                axisSpec.extent = ChartAxesUtils.getMeasureExtent(chartData, axisSpec, isLogScale, includeZero, true, otherAxesIndexes);
 
                 if (axisSpec.extent === null) {
                     return null;
@@ -543,8 +544,8 @@
                     }
 
                     const canHaveAxisTitle = ChartFeatures.canShowXAxisTitle(chartDef) && ((!_.isNil(chartDef.xAxisFormatting.axisTitle) && chartDef.xAxisFormatting.axisTitle.length > 0)
-                    || (xAxis && (xAxis.dimension !== undefined || xAxis.measure !== undefined))
-                    || chartDef.genericDimension0.length === 1);
+                        || (xAxis && (xAxis.dimension !== undefined || xAxis.measure !== undefined))
+                        || ChartDimension.getGenericDimensions(chartDef).length === 1);
 
                     if (canHaveAxisTitle && chartDef.xAxisFormatting.showAxisTitle) {
                         margins.bottom += ChartsStaticData.AXIS_MARGIN + chartDef.xAxisFormatting.axisTitleFormatting.fontSize;
@@ -863,10 +864,10 @@
                     }
 
                     if (chartDef.valuesInChartDisplayOptions.displayValues && chartDef.type === CHART_TYPES.GROUPED_COLUMNS) {
-                        svc.adjustVerticalBarsYAxisPadding(axis, vizHeight, ChartLabels.getMeasureValueInChartMaxHeight(chartDef, true, false, 2, computeTextHeightFromFontSize($svgs)), true);
+                        svc.adjustVerticalBarsYAxisPadding(axis, vizHeight, ChartLabels.getMeasureValueInChartMaxHeight(chartDef, true, false, 2, computeTextHeightFromFontSize($svgs)), true, chartDef);
                     }
 
-                    if (chartDef.valuesInChartDisplayOptions.displayValues && ChartFeatures.canDisplayTotalValues(chartDef)) {
+                    if (chartDef.valuesInChartDisplayOptions.displayValues && ChartFeatures.canDisplayTotalValues(chartDef) && !chartDef.genericMeasures.some(m => ChartMeasure.isRealUnaggregatedMeasure(m))) {
                         svc.adjustVerticalBarsYAxisPadding(axis, vizHeight, ChartLabels.getMeasureValueInChartMaxHeight(chartDef, false, true, 5, computeTextHeightFromFontSize($svgs)), true);
                     }
 
@@ -991,7 +992,7 @@
 
                     g.attr('transform', 'translate(' + updatedMargins.left + ',' + updatedMargins.top + ')');
 
-                    if (xAxis && chartDef.gridlinesOptions.vertical.show && ChartFeatures.canHaveVerticalGridlines(chartDef)) {
+                    if (xAxis && chartDef.gridlinesOptions.vertical.show && ChartFeatures.canHaveVerticalGridlines(chartDef) && !chartHandler.noXAxis) {
                         const gridlinesPosition = { x1: 0, x2: 0, y1: 0, y2: allNegative ? vizHeight : -vizHeight };
                         svc.drawGridlines(g.select('g.x.axis'), 'x', chartDef.gridlinesOptions.vertical.lineFormatting, gridlinesPosition, g);
                     }
@@ -1075,7 +1076,7 @@
              * @param {string} title: title to display
              * @param {number} maxTextWidth: maximum width for the graphical text element
              */
-            formatAxisTitle: function(labelElement, title, maxTextWidth){
+            formatAxisTitle: function(labelElement, title, maxTextWidth) {
                 if (maxTextWidth > 0) {
                     let textLength = labelElement.node().getComputedTextLength();
                     let displayedText = title;
@@ -1093,7 +1094,7 @@
              * @param {string} title: title to display
              * @param {d3 axis}axis : d3 axis for which we want to display the title
              */
-            addDateToTitle: function(title, axis){
+            addDateToTitle: function(title, axis) {
                 let finalTitle = title;
                 if (title && axis.dimension && ChartUADimension.isDate(axis.dimension) && axis.scaleType === SCALE_TYPES.TIME) {
                 // specific case for a date dimension, if the current domain is contained within a single day, we add this date at the end of the title
@@ -1144,6 +1145,9 @@
                             .on('click', function() {
                                 chartHandler.openSection(ChartFormattingPaneSections.X_AXIS);
                             });
+                        chartHandler.$on('$destroy', () => {
+                            labelElement.on('click', null);
+                        });
                     }
 
                     const bbox = labelElement.node().getBoundingClientRect();
@@ -1206,6 +1210,9 @@
                             .on('click', function() {
                                 chartHandler.openSection(ChartFormattingPaneSections.Y_AXIS);
                             });
+                        chartHandler.$on('$destroy', () => {
+                            labelElement.on('click', null);
+                        });
                     }
                 }
             },
@@ -1246,9 +1253,28 @@
              * @param {number} valuesFontSize - the font size of the values displayed in chart
              * @returns {Record<string, unknown>} the updated d3 axis object.
              */
-            adjustVerticalBarsYAxisPadding: (axis, axisHeight, valuesFontSize, addNegativeExtent = true) => {
+            adjustVerticalBarsYAxisPadding: (axis, axisHeight, valuesFontSize, addNegativeExtent, chartDef) => {
                 const extent = svc.getCurrentAxisExtent(axis);
-                adjustAxisPadding(axis, extent, axisHeight, valuesFontSize, extent[0] < 0 && addNegativeExtent, extent[1] > 0);
+                if (chartDef?.variant === CHART_VARIANTS.waterfall && chartDef.genericMeasures.length > 0) {
+                    const valuePlacement = chartDef.genericMeasures[0].valuesInChartDisplayOptions?.placementMode;
+                    switch (valuePlacement) {
+                        case ValuesInChartPlacementMode.BELOW:
+                            adjustAxisPadding(axis, extent, axisHeight, valuesFontSize, true, false);
+                            break;
+                        case ValuesInChartPlacementMode.ABOVE:
+                            adjustAxisPadding(axis, extent, axisHeight, valuesFontSize, false, true);
+                            break;
+                        case ValuesInChartPlacementMode.AUTO:
+                            adjustAxisPadding(axis, extent, axisHeight, valuesFontSize, extent[0] < 0, extent[1] > 0);
+                            break;
+
+                        default:
+                            break;
+                    }
+
+                } else {
+                    adjustAxisPadding(axis, extent, axisHeight, valuesFontSize, extent[0] < 0 && addNegativeExtent, extent[1] > 0);
+                }
                 return axis;
             },
 
@@ -1272,7 +1298,7 @@
              * @param {number} axisSize (in pixel)
              * @returns {number} the optimum ticks number
              */
-            optimizeTicksNumber: function(axisSize){
+            optimizeTicksNumber: function(axisSize) {
                 return Math.floor(Math.max(axisSize / 30, 2));
             },
 
