@@ -8,7 +8,7 @@
     /**
      * (!) This service previously was in static/dataiku/js/simple_report/common/tooltips.js
      */
-    function ChartTooltips($q, $templateCache, $http, $compile, $rootScope, ChartFilters, ChartFormatting, ChartColorScales, ChartLabels, ChartDimension, ChartColorUtils, AnimatedChartsUtils, CHART_TYPES, ChartStoreFactory, ChartFeatures, Logger, ChartUADimension, $stateParams, DKU_OTHERS_VALUE, DashboardFilters, DashboardUtils, HierarchicalChartsUtils, ConditionalFormattingOptions) {
+    function ChartTooltips($q, $templateCache, $http, $compile, $rootScope, ChartFilters, ChartFormatting, ChartColorScales, ChartLabels, ChartDimension, ChartColorUtils, AnimatedChartsUtils, CHART_TYPES, ChartStoreFactory, ChartFeatures, Logger, ChartUADimension, $stateParams, DKU_OTHERS_VALUE, DashboardFilters, DashboardUtils, HierarchicalChartsUtils, ConditionalFormattingOptions, ChartDrilldown, ChartHierarchyDimension, DKU_TOTAL_VALUE, DRILL_UP_SOURCE, DRILL_DOWN_SOURCE) {
         const DASHBOARD_TOOLTIPS_CONTAINER_SELECTOR = '.dashboard-tile__chart-tooltips-container';
         const DASHBOARD_TOOLTIP_CONTAINER_CLASS = 'dashboard-tile__chart-tooltip-container';
         const DASHBOARD_TOOLTIP_CONTAINER_SELECTOR = `.${DASHBOARD_TOOLTIP_CONTAINER_CLASS}`;
@@ -16,8 +16,10 @@
         const ECHARTS_CHART_CONTAINER_SELECTOR = '.main-echarts-zone > .chart';
         const D3_CHART_CONTAINER_SELECTOR = '.mainzone > .charts';
         const CHART_WRAPPER_SELECTOR = '.chart-wrapper';
+        const DEFAULT_COLOR_TRANSITION_DURATION = 300;
 
         let globalTooltipScopes = 0;
+        let resetColorsTimeout = null;
 
         /**
          * Retrieves the tooltip container element.
@@ -145,7 +147,6 @@
                 tooltipScope.facetIndex = index;
                 tooltipScope.tooltipState = { ...defaultTooltipState };
                 tooltipScope.dimensionEntries = [];
-                tooltipScope.getMeasureLabel = ChartLabels.getLongMeasureLabel;
                 tooltipScope.getDimensionLabel = ChartLabels.getDimensionLabel;
                 tooltipScope.getDimensionNumberFormattingOptions = ChartDimension.getNumberFormattingOptions.bind(ChartDimension);
                 tooltipScope.isInDashboard = isInDashboard;
@@ -175,6 +176,16 @@
                         xDimensionIds = chartDef.xDimension.map(xDim => chartStore.getDimensionId(xDim)).filter(dim => dim !== undefined),
                         yDimensionIds = chartDef.yDimension.map(yDim => chartStore.getDimensionId(yDim)).filter(dim => dim !== undefined);
 
+                    const currentHierarchyXDim = ChartHierarchyDimension.getCurrentHierarchyDimension(chartDef, 'x');
+                    const currentHierarchyYDim = ChartHierarchyDimension.getCurrentHierarchyDimension(chartDef, 'y');
+
+                    if (currentHierarchyXDim) {
+                        xDimensionIds.push(chartStore.getDimensionId(currentHierarchyXDim));
+                    }
+                    if (currentHierarchyYDim) {
+                        yDimensionIds.push(chartStore.getDimensionId(currentHierarchyYDim));
+                    }
+
                     const getColorContext = (colorOptions, colorMeasureIndex) => ({
                         chartData: chartData,
                         colorOptions: colorOptions,
@@ -267,106 +278,6 @@
                 tooltipScope.canMultiDimensionalFilter = false;
                 tooltipScope.canFilter = false;
 
-                tooltipScope.shouldDisplayGenericMeasures = () => (
-                    chartDef.genericMeasures.length && !([CHART_TYPES.PIVOT_TABLE, CHART_TYPES.RADAR, CHART_TYPES.SCATTER_MULTIPLE_PAIRS].includes(chartDef.type))
-                );
-
-                tooltipScope.shouldDisplayColorMeasure = () => (
-                    chartDef.colorMeasure.length &&
-                    (ChartFeatures.canHaveConditionalFormatting(chartDef.type) ? chartDef.colorMode !== 'COLOR_GROUPS' : true)
-                );
-
-                tooltipScope.getColorMeasureValue = (tooltipState, colorMeasure, coords, measureFormattingOptions) => {
-                    const colorMeasureIdx = colorMeasure.$mIdx;
-
-                    if ( // avoid errors when tooltip is rendered before a new request is finished
-                        _.isNil(colorMeasureIdx) ||
-                        colorMeasureIdx < 0 || colorMeasureIdx >= tooltipState.formatters.length ||
-                        (chartData.data.aggregations && (colorMeasureIdx >= chartData.data.aggregations.length))
-                    ) {
-                        return '';
-                    }
-
-                    if (!chartData.isBinMeaningful(colorMeasure[0].function, coords, colorMeasureIdx)) {
-                        return tooltipState.formatters[colorMeasureIdx](ChartLabels.NO_VALUE);
-                    }
-
-                    return tooltipState.formatters[colorMeasureIdx](chartData.aggr(colorMeasureIdx).get(coords), measureFormattingOptions);
-                };
-
-                tooltipScope.getAdditionalMeasureValue = (tooltipState, additionalMeasure) => {
-                    return tooltipState.formatters[0](additionalMeasure.value, _.get(chartDef, additionalMeasure.chartDefPath));
-                };
-
-                tooltipScope.canHaveConditionalFormattingTooltips = (coords) => {
-                    // remove animation and facet from coords length
-                    const isTotal = (Object.keys(coords).length - 2) < Object.keys(chartData.axesDef).length;
-
-                    return !isTotal && ChartFeatures.canHaveConditionalFormatting(chartDef.type);
-                };
-
-                tooltipScope.getMessage = (coords) => {
-                    if (chartDef.type === CHART_TYPES.PIVOT_TABLE && !tooltipScope.canHaveConditionalFormattingTooltips(coords)) {
-                        return 'Color and value details are not supported for subtotals and totals';
-                    }
-
-                    return '';
-                };
-
-                /**
-                 * get colors for colorGroups in tooltips
-                 * @param {ChartDef.ColorGroup} colorGroup
-                 * @param {Record<string, string>} coords
-                 * @return {string}
-                 */
-                tooltipScope.getTooltipColorGroupColor = (colorGroup, coords) => {
-                    const measureIndex = colorGroup.basedOnMeasureIndex;
-                    if (measureIndex < 0 || measureIndex >= chartData.data.aggregations.length) {
-                        // avoid error when chartData does not have yet all aggregations
-                        return '';
-                    }
-
-                    let colorMeasureValue = chartData.aggr(measureIndex).get(coords);
-
-                    if (colorGroup.colorGroupMode === 'SCALE') {
-                        if (!chartData.isBinMeaningful(colorGroup.basedOnMeasure.function, coords, measureIndex)) {
-                            return 'transparent';
-                        }
-
-                        return colorGroup.colorProperties.getColorScale()(colorMeasureValue, colorGroup.colorProperties.getBin(coords));
-                    } else if (colorGroup.colorGroupMode === 'RULES') {
-                        const measureRules = colorGroup.rules;
-                        const colorMeasure = colorGroup.basedOnMeasure;
-
-                        if (!chartData.isBinMeaningful(colorGroup.basedOnMeasure.function, coords, measureIndex)) {
-                            colorMeasureValue = '';
-                        }
-
-                        const ruleClass = ConditionalFormattingOptions.getColorRuleClass(colorMeasureValue, measureRules, colorMeasure, chartHandler.getChartTheme());
-
-                        return ConditionalFormattingOptions.getStripeColor(ruleClass.class, ruleClass.customColors) || 'transparent';
-                    }
-
-                    return 'transparent';
-                };
-
-                tooltipScope.formatUnaggregatedAxisValues = function(uaDimension, chartType, data, dataIndex, coords) {
-                    if (chartData.uaFormattedAxisValues) {
-                        let axisValues;
-                        if (chartType === CHART_TYPES.SCATTER_MULTIPLE_PAIRS) {
-                            axisValues = data.axesPairs[coords.extras.pair].values['tooltip_' + dataIndex];
-                        } else {
-                            axisValues = data.values['tooltip_' + dataIndex];
-                        }
-                        return chartData.uaFormattedAxisValues(uaDimension, axisValues, coords.x);
-                    }
-                    return 'NA';
-                };
-
-                tooltipScope.getMeasureTooltipId = function(isLabel, measureName) {
-                    return 'qa_charts_tooltip-measure-' + (measureName || '').replace(/\s/g, '_') + (isLabel ? '-name' : '-value');
-                };
-
                 tooltipScope.coords = function() {
                     return angular.extend({}, tooltipScope.tooltipState.coords, { facet: facetIndex });
                 };
@@ -393,7 +304,7 @@
 
                 globalTooltipScopes++;
 
-                tooltipScope.$on('$destroy', function(){
+                tooltipScope.$on('$destroy', function() {
                     globalTooltipScopes--;
                     Logger.debug('Destroying tooltipScope: ' + tooltipScopes.$id + ' - new total ' + globalTooltipScopes);
                     tooltipScopes[facetIndex] = null;
@@ -411,10 +322,6 @@
             };
         };
 
-        const hasCustomXYAxesDef = (chartDef) => {
-            return [CHART_TYPES.PIVOT_TABLE, CHART_TYPES.SANKEY, CHART_TYPES.TREEMAP].includes(chartDef.type);
-        };
-
         const hasUaDimensions = (chartDef) => {
             return chartDef.type === CHART_TYPES.SCATTER;
         };
@@ -429,6 +336,39 @@
             ];
         };
 
+        const unfocusElement = function(desaturatedColor) {
+            const d3Elem = d3.select(this);
+            if (d3Elem.datum().transparent) {
+                d3Elem.interrupt('updateFocus').transition('updateFocus').duration(DEFAULT_COLOR_TRANSITION_DURATION).attr('opacity', 0);
+            } else {
+                d3Elem.interrupt('updateFocus').transition('updateFocus').duration(DEFAULT_COLOR_TRANSITION_DURATION).attr(this._colorAttr, desaturatedColor);
+            }
+        };
+
+        const focusElement = function(color, chartTransparency) {
+            const d3Elem = d3.select(this);
+            if (d3Elem.datum().transparent) {
+                d3Elem.interrupt('updateFocus').transition('updateFocus').duration(DEFAULT_COLOR_TRANSITION_DURATION)
+                    .attr('opacity', chartTransparency)
+                    .attr(this._colorAttr, color);
+            } else {
+                d3Elem.transition(DEFAULT_COLOR_TRANSITION_DURATION)
+                    .attr(this._colorAttr, color);
+            }
+        };
+
+        const resetElementColor = function(color) {
+            const d3Elem = d3.select(this);
+            if (d3Elem.datum().transparent) {
+                d3Elem.interrupt('updateFocus').transition('updateFocus').duration(DEFAULT_COLOR_TRANSITION_DURATION)
+                    .attr('opacity', 0)
+                    .attr(this._colorAttr, color);
+            } else {
+                d3Elem.interrupt('updateFocus').transition('updateFocus').duration(DEFAULT_COLOR_TRANSITION_DURATION)
+                    .attr(this._colorAttr, color);
+            }
+        };
+
         return {
             /**
              * Initialize the tooltip(s) for the given chart and return an object with methods to control them, and enable tooltips on new elements.
@@ -437,6 +377,7 @@
              * @param {$scope} chartHandler
              * @param {ChartTensorDataWrapper} chartData
              * @param {ChartDef} chartDef
+             * @param {string} colorSpecId
              * @param {array} measureFormatters
              * @param {Function} colorScale
              * @return {{showForCoords: showForCoords, hide: hide, fix: fix, unfix: unfix, setAnimationFrame: setAnimationFrame, getAnimationContext: getAnimationContext, registerEl: registerEl, addTooltipHandlers: addTooltipHandlers, removeTooltipsHandlers: removeTooltipsHandlers, focusColor: focusColor, resetColors: resetColors}}
@@ -461,6 +402,9 @@
                 }
 
                 function getDimensionTooltipEntry(chartDef, chartData, coords, dimension, axisName, colored) {
+                    if (!coords || !axisName) {
+                        return {};
+                    }
                     let label = ChartLabels.getDimensionLabel(dimension);
                     let axisElt = {};
                     // Charts that cannot have binning options also cannot display their values as binned. To prevent that, we set the min and max to null.
@@ -469,16 +413,16 @@
                         axisElt.max = null;
                     }
 
-                    const labelId = 'qa_charts_tooltip-dimension-' + (dimension.column || '').replace(/\s/g, '_') + '-name';
-                    const valueId = 'qa_charts_tooltip-dimension-' + (dimension.column || '').replace(/\s/g, '_') + '-value';
+                    const dimensionTooltipId = `qa_charts_tooltip-dimension-${(dimension.column || '').replace(/\s/g, '_')}`;
 
                     let value, filterValue;
+
                     if (chartDef.type === CHART_TYPES.SCATTER || chartDef.type === CHART_TYPES.SCATTER_MULTIPLE_PAIRS) {
                         label = ChartLabels.getUaLabel(dimension);
                         value = chartData.uaFormattedVal(dimension, coords, axisName);
                         filterValue = value;
                     } else if (chartDef.type === CHART_TYPES.RADAR) {
-                        const radarDimension = chartDef.genericDimension0[0];
+                        const radarDimension = ChartDimension.getGenericDimension(chartDef);
                         // Format labels other than first row
                         label = dimension.isA === 'measure' ?
                             ChartLabels.getLongMeasureLabel(dimension) : // formatting for text
@@ -502,7 +446,10 @@
                             dimension.value,
                             formattingOptions
                         );
-                        filterValue = dimension.label;
+                        filterValue = ChartLabels.getFormattedLabel(
+                            { label: dimension.label },
+                            dimension.isA === 'measure' ? formattingOptions : {}
+                        );
                         axisElt = dimension;
                         dimension = radarDimension;
                     } else {
@@ -517,38 +464,63 @@
                         filterValue = value;
                     }
 
-                    const canChartTypeBeFiltered = ChartFeatures.canFilterInTooltip(chartDef.type);
+                    const canChartTypeBeFiltered = ChartFeatures.canFilterInTooltip(chartDef);
                     const isInDashboard = DashboardUtils.isInDashboard();
                     const isRadarChartMeasure = chartDef.type === CHART_TYPES.RADAR && axisElt.isA === 'measure';
                     const isInReusableDimensionPreview = chartDef.$isInReusableDimensionPreview;
                     const isOthersBin = axisElt.label === DKU_OTHERS_VALUE;
+                    const isTotalBar = axisElt.label === DKU_TOTAL_VALUE;
                     const canAddEmptyFilter = canChartTypeBeFiltered && !isRadarChartMeasure && !isInReusableDimensionPreview;
-                    const canFilter = canChartTypeBeFiltered && !isRadarChartMeasure && !isOthersBin && (!isInDashboard || DashboardFilters.canCrossFilter($stateParams.pageId)) && !isInReusableDimensionPreview;
+                    const canFilter = canChartTypeBeFiltered && !isRadarChartMeasure && !isOthersBin && !isTotalBar && (!isInDashboard || DashboardFilters.canCrossFilter($stateParams.pageId)) && !isInReusableDimensionPreview;
                     const isDrillable = canFilter && !isInDashboard && ChartDimension.isDrillable(dimension);
+                    const nativeDrilldownType = isDrillable ? ChartDimension.isGroupedNumerical(dimension) ? 'NUMERICAL' : 'DATE' : null;
+                    const isHierarchyDrillable = dimension && !isTotalBar && ChartFeatures.canDrillHierarchy(chartDef.type) && dimension.hierarchyId && !isOthersBin && !isRadarChartMeasure;
+                    const isHierarchyDrillableDown = isHierarchyDrillable && ChartHierarchyDimension.canDimensionDrillDown(chartDef, dimension);
+                    const isHierarchyDrillableUp = isHierarchyDrillable && ChartHierarchyDimension.canDimensionDrillUp(chartDef, dimension);
+                    const hierarchyName = !isRadarChartMeasure && ChartHierarchyDimension.getDimensionHierarchyName(chartDef, dimension);
+                    const previousHierarchyDimension = ChartHierarchyDimension.getDimensionHierarchyPreviousDimensionName(chartDef, dimension);
+                    const nextHierarchyDimension = ChartHierarchyDimension.getDimensionHierarchyNextDimensionName(chartDef, dimension);
                     const tooltipLabel = `${ChartLabels.getDimensionLabel(dimension)} - ${filterValue}`;
+                    const chartTheme = chartHandler.getChartTheme();
                     return {
                         label,
                         value,
-                        labelId,
-                        valueId,
+                        labelId: `${dimensionTooltipId}-name`,
+                        valueId: `${dimensionTooltipId}-value`,
+                        drillDownActionId: `${dimensionTooltipId}-drill-down`,
+                        drillUpActionId: `${dimensionTooltipId}-drill-up`,
                         colored,
                         tooltipLabel,
+                        filterValue,
                         isDrillable,
+                        nativeDrilldownType,
+                        isHierarchyDrillableDown,
+                        isHierarchyDrillableUp,
                         canFilter,
+                        hierarchyName,
+                        previousHierarchyDimension,
+                        nextHierarchyDimension,
+                        hasActions: canFilter || (canAddEmptyFilter && !isInDashboard) || isHierarchyDrillableDown || isHierarchyDrillableUp,
                         canAddEmptyFilter,
                         isOthersBin,
+                        chartTheme,
+                        isTotalBar,
                         addEmptyFilter: () => {
                             const newFilter = ChartFilters.createFilter({ column: dimension.column, columnType: dimension.type, excludeOtherValues: false });
                             chartDef.filters = ChartFilters.updateOrAddFilter(chartDef.filters, newFilter);
                         },
                         drill: () => {
-                            const { oldFilter, newFilter, newDateRange } = ChartFilters.getDrillFilter(dimension, axisElt, chartDef.filters);
-
+                            const { updatedFilters, newDateRange } = ChartDrilldown.getUpdateForNativeDrillDown(chartDef, axisElt, dimension);
                             if (newDateRange) {
                                 dimension.dateParams.mode = newDateRange;
                             }
-
-                            chartDef.filters = ChartFilters.updateOrAddFilter(chartDef.filters, newFilter, oldFilter);
+                            chartDef.filters = updatedFilters;
+                        },
+                        hierarchyDrillDown: () => {
+                            ChartDrilldown.handleDrillDown(chartDef, dimension, DRILL_DOWN_SOURCE.TOOLTIP, axisElt);
+                        },
+                        hierarchyDrillUp: () => {
+                            ChartDrilldown.handleDrillUp(chartDef, dimension, DRILL_UP_SOURCE.TOOLTIP);
                         },
                         includeOnly: () => {
                             includeOnly(chartDef, [{ axisElt, dimension, value: filterValue }]);
@@ -567,7 +539,7 @@
                     /** @type {ChartStore} */
                     const chartStore = ChartStoreFactory.get(chartDef.$chartStoreId);
                     const dimensionDefs = [
-                        { isDisplayed: isColorDimensionDisplayed(chartDef, chartData, options) && coords, dimension: ChartColorUtils.getColorDimensionOrMeasure(chartDef), axisName: 'color', colored: isColored(chartDef) },
+                        { isDisplayed: isColorDimensionDisplayed(chartDef, chartData, options) && !!coords, dimension: ChartColorUtils.getColorDimensionOrMeasure(chartDef), axisName: 'color', colored: isColored(chartDef) },
                         { isDisplayed: !!chartDef.uaXDimension[0] && hasUaDimensions(chartDef), dimension: chartDef.uaXDimension[0], axisName: 'x' },
                         { isDisplayed: !!chartDef.uaYDimension[0] && hasUaDimensions(chartDef), dimension: chartDef.uaYDimension[0], axisName: 'y' },
                         { isDisplayed: !!chartDef.uaShape[0] && hasUaDimensions(chartDef), dimension: chartDef.uaShape[0], axisName: 'shape' },
@@ -580,10 +552,30 @@
                         { isDisplayed: chartDef.groupDimension.length, dimension: chartDef.groupDimension[0], axisName: 'group' },
                         { isDisplayed: chartData.axesDef && chartData.axesDef.facet, dimension: chartDef.facetDimension[0], axisName: 'facet' },
                         { isDisplayed: chartData.axesDef && chartData.axesDef.animation, dimension: chartDef.animationDimension[0], axisName: 'animation' },
-                        { isDisplayed: chartDef.xDimension.length && !hasCustomXYAxesDef(chartDef), dimension: chartDef.xDimension[0], axisName: 'x' },
-                        { isDisplayed: chartDef.yDimension.length && !hasCustomXYAxesDef(chartDef), dimension: chartDef.yDimension[0], axisName: 'y' },
+                        { isDisplayed: chartDef.xDimension.length && !ChartDimension.hasCustomDimId(chartDef.type), dimension: chartDef.xDimension[0], axisName: 'x' },
+                        { isDisplayed: chartDef.yDimension.length && !ChartDimension.hasCustomDimId(chartDef.type), dimension: chartDef.yDimension[0], axisName: 'y' },
                         { isDisplayed: chartDef.boxplotBreakdownDim.length && coords, dimension: chartDef.boxplotBreakdownDim[0], axisName: 'x' },
 
+                        ...(chartDef.genericHierarchyDimension.length && ![CHART_TYPES.PIE, CHART_TYPES.RADAR, CHART_TYPES.SANKEY].includes(chartDef.type) ?
+                            chartDef.genericHierarchyDimension[0].dimensions.map(dim => getHierarchyDimensionDef(dim, chartDef.type === CHART_TYPES.STACKED_BARS ? 'y' : 'x', coords))
+                            : []),
+
+                        ...(chartDef.xHierarchyDimension.length && !ChartDimension.hasCustomDimId(chartDef.type) ?
+                            chartDef.xHierarchyDimension[0].dimensions.map(dim => getHierarchyDimensionDef(dim, 'x', coords))
+                            : []),
+
+                        ...(chartDef.yHierarchyDimension.length && !ChartDimension.hasCustomDimId(chartDef.type) ?
+                            chartDef.yHierarchyDimension[0].dimensions.map(dim => getHierarchyDimensionDef(dim, 'y', coords))
+                            : []),
+
+                        ...(chartDef.groupHierarchyDimension.length ?
+                            chartDef.groupHierarchyDimension[0].dimensions.map(dim => getHierarchyDimensionDef(dim, 'group', coords))
+                            : []),
+
+                        ...(chartDef.boxplotBreakdownHierarchyDimension.length ?
+                            chartDef.boxplotBreakdownHierarchyDimension[0].dimensions.map(dim => getHierarchyDimensionDef(dim, 'x', coords))
+                            : []),
+
                         ...(chartDef.type === CHART_TYPES.SCATTER_MULTIPLE_PAIRS) ? getScatterMPDimensions(chartDef, coords) : [],
 
                         chartDef.type === CHART_TYPES.RADAR && { // first row for radar chart
@@ -608,14 +600,10 @@
                                 axisName: 'radarSeries'
                             })) : []),
 
-                        ...(hasCustomXYAxesDef(chartDef) ? chartDef.xDimension.map(xDimension => {
-                            return {
-                                isDisplayed: coords[chartStore.getDimensionId(xDimension)] !== undefined,
-                                dimension: xDimension,
-                                axisName: chartStore.getDimensionId(xDimension)
-                            };
+                        ...(ChartDimension.hasCustomDimId(chartDef.type) ? chartDef.xDimension.map(xDimension => {
+                            return getCustomDimensionDef(chartStore, coords, xDimension);
                         }) : []),
-                        ...(hasCustomXYAxesDef(chartDef) ? chartDef.yDimension.map(yDimension => {
+                        ...(ChartDimension.hasCustomDimId(chartDef.type) ? chartDef.yDimension.map(yDimension => {
                             const yDimensionId = chartStore.getDimensionId(yDimension);
 
                             return {
@@ -623,6 +611,12 @@
                                 dimension: yDimension,
                                 axisName: yDimensionId
                             };
+                        }) : []),
+                        ...(ChartDimension.hasCustomDimId(chartDef.type) && chartDef.xHierarchyDimension && chartDef.xHierarchyDimension.length ? chartDef.xHierarchyDimension[0].dimensions.map(dimension => {
+                            return getHierarchyDimensionDef(dimension, chartStore.getDimensionId(dimension), coords);
+                        }) : []),
+                        ...(ChartDimension.hasCustomDimId(chartDef.type) && chartDef.yHierarchyDimension && chartDef.yHierarchyDimension.length ? chartDef.yHierarchyDimension[0].dimensions.map(dimension => {
+                            return getHierarchyDimensionDef(dimension, chartStore.getDimensionId(dimension), coords);
                         }) : [])
                     ];
 
@@ -631,6 +625,26 @@
                         .map(({ dimension, axisName, colored }) => getDimensionTooltipEntry(chartDef, chartData, coords, dimension, axisName, colored));
                 }
 
+                const getCustomDimensionDef = (chartStore, coords, dimension) => {
+                    const dimensionId = chartStore.getDimensionId(dimension);
+
+                    return {
+                        isDisplayed: coords[dimensionId] !== undefined,
+                        dimension: dimension,
+                        axisName: dimensionId
+                    };
+                };
+
+                const getHierarchyDimensionDef = (dimension, dimensionId, coords) => {
+                    const dimensionLevel = ChartHierarchyDimension.getDimensionLevel(chartDef, dimension);
+                    const hierarchyLevel = ChartHierarchyDimension.getDimensionHierarchyLevel(chartDef, dimension);
+                    return {
+                        isDisplayed: coords && dimensionLevel === hierarchyLevel && !_.isNil(coords[dimensionId]),
+                        dimension: dimension,
+                        axisName: dimensionId
+                    };
+                };
+
                 /**
                  * renderTooltipScopes is used to update tooltipScopes and then render them (with $rootScope.$apply)
                  * @param {Object} tooltipScopes a set of tooltip scopes
@@ -699,11 +713,11 @@
                             tooltipScope.shown = true;
                             const canPinTooltip = ChartFeatures.canPinTooltip(chartDef.type);
                             tooltipScope.dimensionEntries = getDimensionEntries(coords ? { ...coords, facet: facetIndex } : undefined, options);
+                            const canDisplayTooltipMeasures = tooltipScope.dimensionEntries.every(entry => !entry.isTotalBar);
                             tooltipScope.additionalMeasures = options && options.additionalMeasures !== null ? options.additionalMeasures : [];
                             const hasFilterableAxisElts = tooltipScope.dimensionEntries.some(entry => entry.canFilter);
                             tooltipScope.canMultiDimensionalFilter = tooltipScope.dimensionEntries.every(entry => entry.canFilter);
-                            tooltipScope.canFilter = ChartFeatures.canFilterInTooltip(chartDef.type) && hasFilterableAxisElts;
-                            tooltipScope.measureLabel = tooltipScope.shouldDisplayGenericMeasures() ? tooltipScope.getMeasureLabel(chartDef.genericMeasures[measure]) : null;
+                            tooltipScope.canFilter = ChartFeatures.canFilterInTooltip(chartDef) && hasFilterableAxisElts;
 
                             tooltipScope.tooltipState = {
                                 ...tooltipScope.tooltipState,
@@ -711,7 +725,8 @@
                                 measure,
                                 coords,
                                 color,
-                                canPinTooltip
+                                canPinTooltip,
+                                canDisplayTooltipMeasures
                             };
                         };
 
@@ -876,10 +891,7 @@
                         if (!_.isNil(properties) && !_.isNil(colorFocusHandler)) {
                             const colorIndex = ChartColorUtils.getColorIndex(properties.coords, colorFocusHandler.hasColorDim);
                             if (colorFocusHandler.focusColor && !ret.fixed && colorIndex != null) {
-                                // we need a timeout here to fix the race condition when we use a transition to draw lines/bars/labels/circles...
-                                setTimeout(() => {
-                                    ret.focusColor(colorIndex);
-                                });
+                                ret.focusColor(colorIndex);
                             }
                         }
                     },
@@ -893,9 +905,7 @@
                             if (!_.isNil(properties)) {
                                 const colorIndex = ChartColorUtils.getColorIndex(properties.coords, colorFocusHandler.hasColorDim);
                                 if (colorFocusHandler.focusColor && !ret.fixed && colorIndex != null) {
-                                    setTimeout(() => {
-                                        ret.focusColor(colorIndex);
-                                    });
+                                    ret.focusColor(colorIndex);
                                 }
                             } else {
                                 ret.resetColors();
@@ -951,7 +961,7 @@
                         ret.resetColors();
                     },
 
-                    addChartTooltipAndHighlightHandlers: function(chart, getTargetElementProperties, colorFocusHandler, tooltipOptions){
+                    addChartTooltipAndHighlightHandlers: function(chart, getTargetElementProperties, colorFocusHandler, tooltipOptions) {
                         const element = d3.select(chart);
                         element
                             .on('mouseover', () => ret.handleChartMouseOver(getTargetElementProperties, colorFocusHandler))
@@ -1034,11 +1044,9 @@
                                     ret.resetColors();
                                 }
                             })
-                            .on('mouseenter', function() {
+                            .on('mouseenter', function(elemData) {
                                 if (colorAttr && !ret.fixed) {
-                                    setTimeout(() => {
-                                        ret.focusColor(c);
-                                    });
+                                    ret.focusColor(c, elemData);
                                 }
                             });
                     },
@@ -1095,24 +1103,45 @@
                      * Focus on the given color coordinates (ie desaturate all other colors)
                      * @param {number} colorIndex: the color coordinate
                      */
-                    focusColor: function(colorIndex) {
-                        const legend = chartHandler.legendsWrapper.getLegend(0);
-                        if (!_.isNil(legend)) {
-                            if (legend.focusFnList) {
-                                legend.focusFnList.forEach(currentFocusColorFn => currentFocusColorFn(colorIndex));
-                            }
-                            if (!_.isNil(legend.items)) {
-                                legend.items.forEach(function(item, i) {
-                                    if (i != colorIndex) {
-                                        if (item.elements) {
-                                            item.elements.each(function() {
-                                                d3.select(this).transition(300).attr(this._colorAttr, item.desaturatedColor);
+                    focusColor: function(colorIndex, focusedElementData) {
+                        if (resetColorsTimeout) {
+                            // we dont need to reset colors if we immediately focus on another color
+                            clearTimeout(resetColorsTimeout);
+                            resetColorsTimeout = null;
+                        }
+                        setTimeout(() => {
+                            // we need a timeout here to fix the race condition when we use a transition to draw lines/bars/labels/circles...
+                            const legend = chartHandler.legendsWrapper.getLegend(0);
+                            if (!_.isNil(legend)) {
+                                if (legend.focusFnList) {
+                                    legend.focusFnList.forEach(currentFocusColorFn => currentFocusColorFn(colorIndex));
+                                }
+                                if (!_.isNil(legend.items)) {
+                                    legend.items.forEach(function(item, i) {
+                                        if (i != colorIndex) {
+                                            if (item.elements) {
+                                                item.elements.each(function() {
+                                                    unfocusElement.call(this, item.desaturatedColor);
+                                                });
+                                            }
+                                        } else if (item.elements) {
+                                            item.elements.each(function(elementData) {
+                                                if (focusedElementData) {
+                                                    // hover from the chart
+                                                    if (focusedElementData.isUnaggregated && elementData !== focusedElementData) {
+                                                        // particular case for unaggregated data point, we unfocus also all other elements of the same color
+                                                        return unfocusElement.call(this, item.desaturatedColor);
+                                                    }
+                                                    return focusElement.call(this, item.color, chartDef.colorOptions.transparency);
+                                                }
+                                                // hover from the legend
+                                                return resetElementColor.call(this, item.color);
                                             });
                                         }
-                                    }
-                                });
+                                    });
+                                }
                             }
-                        }
+                        });
                     },
 
 
@@ -1120,21 +1149,26 @@
                      * Unfocus everything
                      */
                     resetColors: function() {
-                        const legend = chartHandler.legendsWrapper.getLegend(0);
-                        if (!_.isNil(legend)) {
-                            if (legend.unfocusFnList) {
-                                legend.unfocusFnList.forEach(currentUnfocusFn => currentUnfocusFn());
-                            }
-                            if (!_.isNil(legend.items)) {
-                                legend.items.forEach(function(item) {
-                                    if (item.elements) {
-                                        item.elements.each(function() {
-                                            d3.select(this).transition(300).attr(this._colorAttr, item.color);
-                                        });
-                                    }
-                                });
-                            }
+                        if (resetColorsTimeout) {
+                            clearTimeout(resetColorsTimeout);
                         }
+                        resetColorsTimeout = setTimeout(() => {
+                            const legend = chartHandler.legendsWrapper.getLegend(0);
+                            if (!_.isNil(legend)) {
+                                if (legend.unfocusFnList) {
+                                    legend.unfocusFnList.forEach(currentUnfocusFn => currentUnfocusFn());
+                                }
+                                if (!_.isNil(legend.items)) {
+                                    legend.items.forEach(function(item) {
+                                        if (item.elements) {
+                                            item.elements.each(function() {
+                                                resetElementColor.call(this, item.color);
+                                            });
+                                        }
+                                    });
+                                }
+                            }
+                        }, 5);
                     },
 
                     getAnimationContext: function() {
