@@ -307,6 +307,12 @@
                 singleColor: '#2678B1',
                 transparency: 0.75
             };
+            const waterfallCategories = [
+                { label: 'Increase', column: 'increase', defaultColor: '#8ABA00' },
+                { label: 'Decrease', column: 'decrease', defaultColor: '#FF3545' },
+                { label: 'Others', column: 'others', defaultColor: '#F4D717' },
+                { label: 'Total', column: 'total', defaultColor: '#00C2DE' }
+            ];
             const DEFAULT_FILL_OPACITY = 0.6;
             const DEFAULT_STROKE_WIDTH = 2;
 
@@ -370,6 +376,7 @@
                 },
 
                 DEFAULT_COLOR_OPTIONS,
+                waterfallCategories,
                 DEFAULT_STROKE_WIDTH,
                 DEFAULT_FILL_OPACITY,
                 MEASURES_PERCENT_MODES: ['PERCENT', 'PERCENTAGE', 'CUMULATIVE_PERCENTAGE'],
@@ -408,7 +415,38 @@
                 DEFAULT_DATE_PART_FILTER_TYPE: createDateMode('PART', 'Date part'),
                 TIMELINE_AND_AUTOMATIC_DATE_MODES: [AUTOMATIC].concat(TIMELINE_DATE_MODES),
                 DATE_MODES_WITH_BACKEND_ONLY: DATE_MODES.concat(BACKEND_ONLY_DATE_MODES),
-                BIN_NUMBER_DEFAULT: buildBinNumberConfiguration('default', 30, 30)
+                BIN_NUMBER_DEFAULT: buildBinNumberConfiguration('default', 30, 30),
+
+                PRIMARY_DIMENSIONS_PROPERTIES: [
+                    'genericDimension0',
+                    'boxplotBreakdownDim',
+                    'xDimension',
+                    'yDimension',
+                    'groupDimension'
+                ],
+
+                SECONDARY_DIMENSIONS_PROPERTIES: [
+                    'genericDimension1',
+                    'animationDimension',
+                    'facetDimension'
+                ],
+
+                HIERARCHY_DIMENSIONS_PROPERTIES: [
+                    'genericHierarchyDimension',
+                    'xHierarchyDimension',
+                    'yHierarchyDimension',
+                    'groupHierarchyDimension',
+                    'boxplotBreakdownHierarchyDimension'
+                ],
+
+                MEASURES_PROPERTIES: [
+                    'genericMeasures',
+                    'colorMeasure',
+                    'sizeMeasure',
+                    'xMeasure',
+                    'yMeasure',
+                    'tooltipMeasures'
+                ]
             };
 
             return svc;
@@ -498,7 +536,7 @@
         })
 
         // mock of chart-dimension.service.ts
-        .factory('ChartDimension', function(ChartsStaticData) {
+        .factory('ChartDimension', function(ChartsStaticData, ChartHierarchyDimension) {
             function findBinNumberOrDefault(chartType, binNumbers) {
                 return binNumbers.find(automaticMaxBin => automaticMaxBin.chartType === chartType) || ChartsStaticData.BIN_NUMBER_DEFAULT;
             }
@@ -541,7 +579,8 @@
             }
 
             function isMainDateAxisAutomatic(chartDef) {
-                return chartDef.genericDimension0.length > 0 && isAutomatic(chartDef.genericDimension0[0]);
+                const genericDimension = getGenericDimension(chartDef);
+                return genericDimension && isAutomatic(genericDimension);
             }
 
             function getMainDateAxisBinningMode(response) {
@@ -576,33 +615,104 @@
                 return isNumerical(dimension) && ((dimension.numParams && dimension.numParams.mode != 'TREAT_AS_ALPHANUM') || dimension.numParams == undefined);
             }
 
+
+            function getDimensionUniqueId(dimension, index, type) {
+                return (`${type}_${index}_${dimension.column}`).replace('.', '_');
+            }
+
+            function isDrillable(dimension) {
+                const isAutomatic = this.isAutomatic(dimension);
+                const isUnaggregated = this.isUnaggregated(dimension);
+
+                return !isAutomatic && !isUnaggregated && ((isTimelineable(dimension) && dimension.dateParams.mode !== "HOUR") ||
+                isGroupedNumerical(dimension));
+            }
+
+            function containsInteractiveDimensionCandidate(chartDef) {
+                const genericDimension = getGenericDimension(chartDef);
+                if (!genericDimension) {
+                    return false;
+                }
+                return isCandidateForInteractivity(genericDimension);
+            }
+
+            function isInteractiveChart(chartDef) {
+                if (chartDef.type !== 'lines') {
+                    return false;
+                }
+                return containsInteractiveDimensionCandidate(chartDef);
+            }
+
+            function isCandidateForInteractivity(dimension) {
+                return isAutomatic(dimension) && !isOrdinalDateScale(dimension);
+            }
+
+            function isGroupedNumerical(dimension) {
+                if (!dimension) {
+                    return;
+                }
+                return isTrueNumerical(dimension) && ((dimension.numParams && dimension.numParams.mode != 'NONE') || dimension.numParams == undefined);
+            }
+
+            function getGenericDimension(chartDef) {
+                return getGenericDimensions(chartDef)[0];
+            }
+
+            function getGenericDimensions(chartDef) {
+                if (ChartHierarchyDimension.canHaveGenericHierarchyDimension(chartDef.type)) {
+                    const hierarchyDimension = ChartHierarchyDimension.getCurrentHierarchyDimension(chartDef)
+                    return hierarchyDimension ? [hierarchyDimension] : chartDef.genericDimension0;
+                }
+                return chartDef.genericDimension0;
+            }
+
+            function getXDimension(chartDef) {
+                return getXDimensions(chartDef)[0];
+            }
+
+            function getXDimensions(chartDef) {
+                if (ChartHierarchyDimension.canHaveXHierarchyDimension(chartDef.type)) {
+                    const hierarchyDimension = ChartHierarchyDimension.getCurrentHierarchyDimension(chartDef, 'x');
+                    return hierarchyDimension ? [hierarchyDimension] : chartDef.xDimension;
+                }
+                return chartDef.xDimension;
+            }
+
+            function getYDimension(chartDef) {
+                return getYDimensions(chartDef)[0];
+            }
+
+            function getYDimensions(chartDef) {
+                if (ChartHierarchyDimension.canHaveYHierarchyDimension(chartDef.type)) {
+                    const hierarchyDimension = ChartHierarchyDimension.getCurrentHierarchyDimension(chartDef, 'y');
+                    return hierarchyDimension ? [hierarchyDimension] : chartDef.yDimension;
+                }
+                return chartDef.yDimension;
+            }
+
             return {
                 isTimelineable,
                 isUnaggregated,
                 isAutomatic,
+                isDrillable,
                 getNumberFormattingOptions,
                 hasFixedNumberOfBins,
                 hasFixedBinSize,
                 isNumerical,
+                getDimensionUniqueId,
                 isTrueNumerical,
+                containsInteractiveDimensionCandidate,
+                isInteractiveChart,
+                isCandidateForInteractivity,
+                isGroupedNumerical,
+                getXDimension,
+                getXDimensions,
+                getYDimension,
+                getYDimensions,
+                getGenericDimension,
                 isTimeline: function(dimension) {
                     return !isOrdinalDateScale(dimension) && isTimelineable(dimension);
                 },
-                containsInteractiveDimensionCandidate: function(chartDef) {
-                    if (chartDef.genericDimension0.length === 0) {
-                        return false;
-                    }
-                    return this.isCandidateForInteractivity(chartDef.genericDimension0[0]);
-                },
-                isInteractiveChart: function(chartDef) {
-                    if (chartDef.type !== 'lines') {
-                        return false;
-                    }
-                    return this.containsInteractiveDimensionCandidate(chartDef);
-                },
-                isCandidateForInteractivity: function(dimension) {
-                    return isAutomatic(dimension) && !isOrdinalDateScale(dimension);
-                },
                 isPercentScale: function(measures) {
                     return measures.every((measure) => ChartsStaticData.MEASURES_PERCENT_MODES.includes(measure.computeMode));
                 },
@@ -650,12 +760,6 @@
                     }
                     return dimension.type == 'ALPHANUM' || (dimension.type == 'NUMERICAL' && dimension.numParams && dimension.numParams.mode == 'TREAT_AS_ALPHANUM');
                 },
-                isGroupedNumerical: function(dimension) {
-                    if (!dimension) {
-                        return;
-                    }
-                    return this.isTrueNumerical(dimension) && ((dimension.numParams && dimension.numParams.mode != 'NONE') || dimension.numParams == undefined);
-                },
                 isUngroupedNumerical: function(dimension) {
                     if (!dimension) {
                         return;
@@ -1047,6 +1151,170 @@
             };
         })
 
+        .factory('ChartHierarchyDimension', function() {
+            const HIERARCHY_DIM_PROPS = ['genericHierarchyDimension', 'xHierarchyDimension', 'yHierarchyDimension', 'groupHierarchyDimension', 'boxplotBreakdownHierarchyDimension'];
+
+            function canHaveGenericHierarchyDimension(chartType) {
+                return [
+                    'grouped_columns', 'stacked_columns', 'stacked_bars', 'radar',
+                    'lines', 'multi_columns_lines', 'pie', 'stacked_area'
+                ].includes(chartType);
+            }
+
+            function canHaveGroupHierarchyDimension(chartType) {
+                return [
+                    'grouped_xy', 'lift'
+                ].includes(chartType);
+            }
+
+            function canHaveXHierarchyDimension(chartType) {
+                return [
+                    'binned_xy', 'pivot_table'
+                ].includes(chartType);
+            }
+
+            function canHaveYHierarchyDimension(chartType) {
+                return [
+                    'binned_xy', 'pivot_table', 'treemap'
+                ].includes(chartType);
+            }
+
+            function canHaveBreakdownHierarchyDimension(chartType) {
+                return chartType === 'boxplots'
+            }
+
+            function getHierarchyDimensionForChartType(chartDef, axis = 'x') {
+                if (canHaveGenericHierarchyDimension(chartDef.type)) {
+                    return chartDef.genericHierarchyDimension;
+                }
+                if (canHaveGroupHierarchyDimension(chartDef.type)) {
+                    return chartDef.groupHierarchyDimension;
+                }
+                if (canHaveBreakdownHierarchyDimension(chartDef.type)) {
+                    return chartDef.boxplotBreakdownHierarchyDimension;
+                }
+                else if (canHaveYHierarchyDimension(chartDef.type) && canHaveXHierarchyDimension(chartDef.type)) {
+                    return axis === 'x' ? chartDef.xHierarchyDimension : axis === 'y' ? chartDef.yHierarchyDimension : undefined;
+                }
+                else if (canHaveXHierarchyDimension(chartDef.type)) {
+                    return chartDef.xHierarchyDimension;
+                }
+                else if (canHaveYHierarchyDimension(chartDef.type)) {
+                    return chartDef.yHierarchyDimension;
+                }
+                return;
+            }
+
+            function getCurrentHierarchyDimension(chartDef, axis = 'x') {
+                const hierarchyDimension = getHierarchyDimensionForChartType(chartDef, axis);
+                if (!hierarchyDimension || !hierarchyDimension.length || !hierarchyDimension[0]) {
+                    return;
+                }
+                return hierarchyDimension[0].dimensions[hierarchyDimension[0].level];
+            }
+
+            function getDimensionHierarchyPropName(chartDef, hierarchyId) {
+                if (!hierarchyId) {
+                    return;
+                }
+                return HIERARCHY_DIM_PROPS.find(prop => {
+                    const hierarchyDim = (chartDef[prop] || [])[0];
+                    return hierarchyDim && hierarchyDim.id === hierarchyId;
+                });
+            }
+
+            function getDimensionHierarchy(chartDef, dimension) {
+                const key = getDimensionHierarchyPropName(chartDef, dimension.hierarchyId);
+                if (key) {
+                    return (chartDef[key] || [])[0];
+                }
+                return;
+            }
+
+            function getDimensionLevel(chartDef, dimension) {
+                const hierarchy = getDimensionHierarchy(chartDef, dimension);
+                return hierarchy?.dimensions.findIndex(hDim => hDim.column === dimension.column);
+            }
+
+            function getHierarchyNextUsableDimensionIdx (_, __, currentLevel) {
+                return currentLevel + 1;
+            }
+
+            function getHierarchyPreviousUsableDimensionIdx (_, __, currentLevel) {
+                return currentLevel ? currentLevel - 1 : 0;
+            }
+
+            function canDimensionDrill(chartDef, dimension, direction) {
+                const hierarchy = getDimensionHierarchy(chartDef, dimension);
+                const dimensionLevel = getDimensionLevel(chartDef, dimension);
+                if (!hierarchy || !_.isNumber(dimensionLevel)) {
+                    return false;
+                }
+                if (direction === 'UP') {
+                    return getHierarchyPreviousUsableDimensionIdx(chartDef.$chartStoreId, hierarchy, dimensionLevel) > -1;
+                } else {
+                    return getHierarchyNextUsableDimensionIdx(chartDef.$chartStoreId, hierarchy, dimensionLevel) > -1;
+                }
+            }
+        
+            function canDimensionDrillUp(chartDef, dimension) {
+                return canDimensionDrill(chartDef, dimension, 'UP');
+            }
+        
+            function canDimensionDrillDown(chartDef, dimension) {
+                return canDimensionDrill(chartDef, dimension, 'DOWN');
+            }
+
+            function getDimensionHierarchyName(chartDef, dimension) {
+                const hierarchy = getDimensionHierarchy(chartDef, dimension);
+                return hierarchy?.name;
+            }
+
+            return {
+                canDimensionDrillUp,
+                canDimensionDrillDown,
+                getDimensionHierarchyName,
+                getDimensionHierarchy,
+                getCurrentHierarchyDimension,
+                canHaveGenericHierarchyDimension,
+                canHaveXHierarchyDimension,
+                canHaveYHierarchyDimension
+            };
+        })
+
+        .factory('ChartDrilldown', function(ChartLabels, ChartHierarchyDimension) {
+            return {
+                getDrillableElements(store, chartDef, chartData, coordsDict) {
+                    const drillDownElements = [];
+            
+                    const possibleHierarchyCoords = ['x', 'y', 'color', 'group'];
+                    const hierarchyCoords = Object.keys(coordsDict).filter(coord => possibleHierarchyCoords.includes(coord) || coord.includes('HierarchyDimension'));
+            
+                    hierarchyCoords.forEach(dimId => {
+                        const dimension = store.get('dimensionDict').get(dimId);
+                        const coord = coordsDict[dimId];
+                        const axis = chartData.getAxisLabels(dimId);
+            
+                        if (dimension && !_.isNil(coord) && axis && !_.isNil(dimension.hierarchyId)) {
+                            const axisElt = axis[coord];
+                            const dimensionLabel = ChartLabels.getDimensionLabel(dimension);
+                            const value = ChartLabels.getAxisEltFormattedLabel({
+                                chartDef,
+                                chartData,
+                                dimension,
+                                axisName: dimId,
+                                axisElt
+                            });
+                            const drilldown = ChartHierarchyDimension.canDimensionDrillDown(chartDef, dimension);
+                            const drillup = ChartHierarchyDimension.canDimensionDrillUp(chartDef, dimension);
+                            drillDownElements.push({ axisElt, dimension, dimensionLabel, value, drilldown, drillup });
+                        }
+                    });
+                    return drillDownElements;
+                }
+            }
+        })
+
         .factory('ChartFilterUtils', function(ChartsStaticData, ChartColumnTypeUtils) {
             function toDateFilterType(dateMode) {
                 let label = dateMode.label;
@@ -1235,6 +1503,10 @@
                 isCount: function(column) {
                     return !!column && (column.column === '__COUNT__' || column.column == null && column.function === 'COUNT');
                 },
+
+                isHierarchy(column) {
+                    return !!column && column.hasOwnProperty('dimensions') && column.hasOwnProperty('name');
+                },
             
                 isNonColumnData: function(column) {
                     return !column || !column.type;
@@ -1272,7 +1544,7 @@
             };
         })
 
-        .factory('ChartColorUtils', function() {
+        .factory('ChartColorUtils', function(ChartDimension) {
             return {
                 getDiscretePalette(colorPaletteId, customPalette, theme) {
                     if (colorPaletteId === '__dku_custom__') {
@@ -1304,11 +1576,11 @@
                         case 'geom_map':
                             return chartDef.uaColor[0];
                         case 'pie':
-                            return chartDef.genericDimension0[0];
+                            return ChartDimension.getGenericDimension(chartDef);
                         case 'pivot_table':
                             return chartDef.colorMeasure;
                         case 'treemap':
-                            return chartDef.colorMeasure[0] || chartDef.yDimension[0];
+                            return ChartDimension.getYDimension(chartDef);
                         default:
                             return chartDef.genericDimension1[0];
                     }
@@ -1381,6 +1653,14 @@
             };
         })
 
+        .factory('ChartDrilldownType', function() {
+            return {
+                HIERARCHY: 'hierarchy',
+                DATE: "date",
+                BIN: "bin"
+            };
+        })
+
         .factory('MultiplotUtils', function() {
             return {
 
@@ -1454,5 +1734,11 @@
                 applyToSankeyNodeLabels: () => {}
             };
         })
+        .factory('Ng2MenuOrchestratorService', function() {
+            return {
+                triggerMenuOpen: () => {
+                }
+            };
+        })
         .factory('DefaultDSSVisualizationTheme', () => defaultTheme)
 })();
